@@ -4,6 +4,7 @@ Authors
  * Peter Plantinga 2020
  * Abdel Heba 2020
  * Mirco Ravanelli 2020
+ * Aku Rouhe 2021
 """
 
 import os
@@ -30,6 +31,7 @@ from torch.utils.data import DistributedSampler
 from torch.nn.parallel import DistributedDataParallel as DDP
 from hyperpyyaml import resolve_references
 from speechbrain.utils.distributed import run_on_main
+from speechbrain.dataio.dataloader import LoopedLoader
 from speechbrain.dataio.dataloader import SaveableDataLoader
 from speechbrain.dataio.sampler import DistributedSamplerWrapper
 from speechbrain.dataio.sampler import ReproducibleRandomSampler
@@ -123,13 +125,14 @@ def _logging_excepthook(exc_type, exc_value, exc_traceback):
     logger.error("Exception:", exc_info=(exc_type, exc_value, exc_traceback))
 
 
-def parse_arguments(arg_list):
+def parse_arguments(arg_list=None):
     r"""Parse command-line arguments to the experiment.
 
     Arguments
     ---------
-    arg_list : list
-        A list of arguments to parse, most often from `sys.argv[1:]`.
+    arg_list : list, None
+        A list of arguments to parse.  If not given, this is read from
+        `sys.argv[1:]`
 
     Returns
     -------
@@ -151,9 +154,9 @@ def parse_arguments(arg_list):
     >>> overrides
     'seed: 10'
     """
-    parser = argparse.ArgumentParser(
-        description="Run a SpeechBrain experiment",
-    )
+    if arg_list is None:
+        arg_list = sys.argv[1:]
+    parser = argparse.ArgumentParser(description="Run a SpeechBrain experiment")
     parser.add_argument(
         "param_file",
         type=str,
@@ -186,9 +189,7 @@ def parse_arguments(arg_list):
         help="A file storing the configuration options for logging",
     )
     # if use_env = False in torch.distributed.lunch then local_rank arg is given
-    parser.add_argument(
-        "--local_rank", type=int, help="Rank on local machine",
-    )
+    parser.add_argument("--local_rank", type=int, help="Rank on local machine")
     parser.add_argument(
         "--device",
         type=str,
@@ -228,7 +229,7 @@ def parse_arguments(arg_list):
     )
     parser.add_argument(
         "--auto_mix_prec",
-        default=False,
+        default=None,
         action="store_true",
         help="This flag enables training with automatic mixed-precision.",
     )
@@ -245,7 +246,7 @@ def parse_arguments(arg_list):
     )
     parser.add_argument(
         "--noprogressbar",
-        default=False,
+        default=None,
         action="store_true",
         help="This flag disables the data loop progressbars.",
     )
@@ -367,10 +368,8 @@ class Brain:
             If a non-positive number is passed, all epochs are run.
         jit_module_keys (list of str)
             List of keys in ``modules`` that should be jit compiled.
-        distributed_count (int)
-            Number of devices to run on.
         distributed_backend (str)
-            One of ``ddp_nccl``, ``ddp_gloo``, ``ddp_mpi``, ``data_parallel``.
+            One of ``nccl``, ``gloo``, ``mpi``.
         device (str)
             The location for performing computations.
         auto_mix_prec (bool)
@@ -387,6 +386,11 @@ class Brain:
         ckpt_interval_minutes (float)
             Amount of time between saving intra-epoch checkpoints,
             in minutes, default: ``15.0``. If non-positive, these are not saved.
+
+        Typically in a script this comes from ``speechbrain.parse_args``, which
+        has different defaults than Brain. If an option is not defined here
+        (keep in mind that parse_args will inject some options by default),
+        then the option is also searched for in hparams (by key).
     checkpointer : speechbrain.Checkpointer
         By default, this will be used to load checkpoints, and will have the
         optimizer added to continue training if interrupted.
@@ -432,11 +436,15 @@ class Brain:
             "noprogressbar": False,
             "ckpt_interval_minutes": 0,
         }
+
         for arg, default in run_opt_defaults.items():
             if run_opts is not None and arg in run_opts:
                 if hparams is not None and arg in hparams:
                     logger.info(
-                        "Info: " + arg + " arg overridden by command line input"
+                        "Info: "
+                        + arg
+                        + " arg overridden by command line input to: "
+                        + str(run_opts[arg])
                     )
                 setattr(self, arg, run_opts[arg])
             else:
@@ -478,7 +486,9 @@ class Brain:
             )
 
         # Switch to the right context
-        if "cuda" in self.device:
+        if self.device == "cuda":
+            torch.cuda.set_device(0)
+        elif "cuda" in self.device:
             torch.cuda.set_device(int(self.device[-1]))
 
         # Put modules on the right device, accessible with dot notation
@@ -544,9 +554,6 @@ class Brain:
                         "Only the main process is alive, "
                         "all other subprocess were killed."
                     )
-            # force the models to start and remain synchronized
-            torch.backends.cudnn.deterministic = True
-            torch.backends.cudnn.benchmark = False
 
         # Prepare iterating variables
         self.avg_train_loss = 0.0
@@ -625,7 +632,7 @@ class Brain:
         pass
 
     def make_dataloader(
-        self, dataset, stage, ckpt_prefix="dataloader-", **loader_kwargs,
+        self, dataset, stage, ckpt_prefix="dataloader-", **loader_kwargs
     ):
         """Creates DataLoaders for Datasets.
 
@@ -679,7 +686,10 @@ class Brain:
         if (
             self.checkpointer is not None
             and ckpt_prefix is not None
-            and isinstance(dataloader, SaveableDataLoader)
+            and (
+                isinstance(dataloader, SaveableDataLoader)
+                or isinstance(dataloader, LoopedLoader)
+            )
         ):
             ckpt_key = ckpt_prefix + stage.name
             self.checkpointer.add_recoverable(ckpt_key, dataloader)
@@ -694,7 +704,7 @@ class Brain:
         if shuffle and not self.distributed_launch:
             if sampler is not None:
                 raise ValueError(
-                    "Cannot specify both shuffle=True "
+                    "Cannot specify both shuffle=True"
                     "and a sampler in loader_kwargs"
                 )
             sampler = ReproducibleRandomSampler(dataset)
@@ -720,27 +730,23 @@ class Brain:
 
                 # with DistributedSamplerWrapper, one must disable shuffling for dataloader
                 loader_kwargs["shuffle"] = False
+                loader_kwargs["sampler"] = self.train_sampler
             elif loader_kwargs.get("batch_sampler") is None:
-                # Currently to get here, shuffle == False, so not passing it.
-                # Otherwise we'd have to handle deleting it (but it is already
-                # deleted).
+                # no sampler and batch-sampler
                 self.train_sampler = DistributedSampler(
-                    dataset,
-                    rank=self.rank,
-                    shuffle=shuffle,
-                    drop_last=drop_last,
+                    dataset, rank=self.rank, shuffle=False, drop_last=drop_last
                 )
 
                 # with DistributedSamplerWrapper, one must disable shuffling for dataloader
                 loader_kwargs["shuffle"] = False
+                loader_kwargs["sampler"] = self.train_sampler
             else:  # batch_sampler was specified
-                # TODO: Could a DistributedSamplerWrapper actually work
-                # just fine for wrapping a BatchSampler, as well?
-                logger.warning(
-                    "Cannot automatically solve distributed sampling "
-                    "when using a BatchSampler."
+                self.train_sampler = DistributedSamplerWrapper(
+                    loader_kwargs.get("batch_sampler", None),
+                    rank=self.rank,
+                    shuffle=False,
                 )
-            loader_kwargs["sampler"] = self.train_sampler
+                loader_kwargs["batch_sampler"] = self.train_sampler
         elif self.distributed_launch and isinstance(dataset, IterableDataset):
             logger.warning(
                 "Cannot automatically solve distributed sampling "
@@ -974,11 +980,17 @@ class Brain:
             Whether to display the progress of each epoch in a progressbar.
         """
 
-        if not isinstance(train_set, DataLoader):
+        if not (
+            isinstance(train_set, DataLoader)
+            or isinstance(train_set, LoopedLoader)
+        ):
             train_set = self.make_dataloader(
                 train_set, stage=sb.Stage.TRAIN, **train_loader_kwargs
             )
-        if valid_set is not None and not isinstance(valid_set, DataLoader):
+        if valid_set is not None and not (
+            isinstance(valid_set, DataLoader)
+            or isinstance(valid_set, LoopedLoader)
+        ):
             valid_set = self.make_dataloader(
                 valid_set,
                 stage=sb.Stage.VALID,
@@ -1035,7 +1047,14 @@ class Brain:
                         and time.time() - last_ckpt_time
                         >= self.ckpt_interval_minutes * 60.0
                     ):
-                        run_on_main(self._save_intra_epoch_ckpt)
+                        # This should not use run_on_main, because that
+                        # includes a DDP barrier. That eventually leads to a
+                        # crash when the processes'
+                        # time.time() - last_ckpt_time differ and some
+                        # processes enter this block while others don't,
+                        # missing the barrier.
+                        if sb.utils.distributed.if_main_process():
+                            self._save_intra_epoch_ckpt()
                         last_ckpt_time = time.time()
 
             # Run train "on_stage_end" on all processes
@@ -1154,7 +1173,10 @@ class Brain:
         if progressbar is None:
             progressbar = not self.noprogressbar
 
-        if not isinstance(test_set, DataLoader):
+        if not (
+            isinstance(test_set, DataLoader)
+            or isinstance(test_set, LoopedLoader)
+        ):
             test_loader_kwargs["ckpt_prefix"] = None
             test_set = self.make_dataloader(
                 test_set, Stage.TEST, **test_loader_kwargs
@@ -1203,10 +1225,7 @@ class Brain:
 
     @sb.utils.checkpoints.mark_as_saver
     def _save(self, path):
-        save_dict = {
-            "step": self.step,
-            "avg_train_loss": self.avg_train_loss,
-        }
+        save_dict = {"step": self.step, "avg_train_loss": self.avg_train_loss}
         with open(path, "w") as w:
             w.write(yaml.dump(save_dict))
 
