@@ -4,6 +4,7 @@ Authors
  * Mirco Ravanelli 2020
  * Jianyuan Zhong 2020
  * Cem Subakan 2021
+ * Davide Borra 2021
 """
 
 import math
@@ -317,6 +318,8 @@ class Conv1d(nn.Module):
         (same, valid, causal). If "valid", no padding is performed.
         If "same" and stride is 1, output shape is the same as the input shape.
         "causal" results in causal (dilated) convolutions.
+    groups: int
+        Number of blocked connections from input channels to output channels.
     padding_mode : str
         This flag specifies the type of padding. See torch.nn documentation
         for more information.
@@ -473,7 +476,7 @@ class Conv1d(nn.Module):
 
 
 class Conv2d(nn.Module):
-    """This function implements 1d convolution.
+    """This function implements 2d convolution.
 
     Arguments
     ---------
@@ -573,7 +576,6 @@ class Conv2d(nn.Module):
 
         """
         x = x.transpose(1, -1)
-
         if self.unsqueeze:
             x = x.unsqueeze(1)
 
@@ -594,9 +596,7 @@ class Conv2d(nn.Module):
 
         if self.unsqueeze:
             wx = wx.squeeze(1)
-
         wx = wx.transpose(1, -1)
-
         return wx
 
     def _manage_padding(
@@ -606,7 +606,7 @@ class Conv2d(nn.Module):
         dilation: Tuple[int, int],
         stride: Tuple[int, int],
     ):
-        """This function performs zero-padding on the time and frequency axises
+        """This function performs zero-padding on the time and frequency axes
         such that their lengths is unchanged after the convolution.
 
         Arguments
@@ -658,6 +658,72 @@ class Conv2d(nn.Module):
         return in_channels
 
 
+class Conv2dWithConstraint(Conv2d):
+    """This function implements 2d convolution with kernel max-norm constaint.
+    This corresponds to set an upper bound for the kernel norm.
+
+    Arguments
+    ---------
+    out_channels : int
+        It is the number of output channels.
+    kernel_size : tuple
+        Kernel size of the 2d convolutional filters over time and frequency
+        axis.
+    input_shape : tuple
+        The shape of the input. Alternatively use ``in_channels``.
+    in_channels : int
+        The number of input channels. Alternatively use ``input_shape``.
+    stride: int
+        Stride factor of the 2d convolutional filters over time and frequency
+        axis.
+    dilation : int
+        Dilation factor of the 2d convolutional filters over time and
+        frequency axis.
+    padding : str
+        (same, valid). If "valid", no padding is performed.
+        If "same" and stride is 1, output shape is same as input shape.
+    padding_mode : str
+        This flag specifies the type of padding. See torch.nn documentation
+        for more information.
+    groups : int
+        This option specifies the convolutional groups. See torch.nn
+        documentation for more information.
+    bias : bool
+        If True, the additive bias b is adopted.
+    max_norm : float
+        kernel  max-norm
+
+    Example
+    -------
+    >>> inp_tensor = torch.rand([10, 40, 16, 8])
+    >>> max_norm = 1
+    >>> cnn_2d_constrained = Conv2dWithConstraint(
+    ...     in_channels=inp_tensor.shape[-1], out_channels=5, kernel_size=(7, 3)
+    ... )
+    >>> out_tensor = cnn_2d_constrained(inp_tensor)
+    >>> torch.any(torch.norm(cnn_2d_constrained.conv.weight.data, p=2, dim=0)>max_norm)
+    tensor(False)
+    """
+
+    def __init__(self, *args, max_norm=1, **kwargs):
+        self.max_norm = max_norm
+        super(Conv2dWithConstraint, self).__init__(*args, **kwargs)
+
+    def forward(self, x):
+        """Returns the output of the convolution.
+
+        Arguments
+        ---------
+        x : torch.Tensor (batch, time, channel)
+            input to convolve. 2d or 4d tensors are expected.
+
+        """
+        self.conv.weight.data = torch.renorm(
+            self.conv.weight.data, p=2, dim=0, maxnorm=self.max_norm
+        )
+        return super(Conv2dWithConstraint, self).forward(x)
+
+
 class ConvTranspose1d(nn.Module):
     """This class implements 1d transposed convolution with speechbrain.
     Transpose convolution is normally used to perform upsampling.
@@ -862,7 +928,7 @@ class ConvTranspose1d(nn.Module):
 
 
 class DepthwiseSeparableConv1d(nn.Module):
-    """This class implements the depthwise separable convolution.
+    """This class implements the depthwise separable 1d convolution.
 
     First, a channel-wise convolution is applied to the input
     Then, a point-wise convolution to project the input to output
@@ -942,7 +1008,7 @@ class DepthwiseSeparableConv1d(nn.Module):
 
 
 class DepthwiseSeparableConv2d(nn.Module):
-    """This class implements the depthwise separable convolution.
+    """This class implements the depthwise separable 2d convolution.
 
     First, a channel-wise convolution is applied to the input
     Then, a point-wise convolution to project the input to output
@@ -1047,14 +1113,16 @@ def get_padding_elem(L_in: int, stride: int, kernel_size: int, dilation: int):
     dilation : int
     """
     if stride > 1:
-        half_kernel = torch.div(kernel_size, 2, rounding_mode="floor")
-        padding = [half_kernel, half_kernel]
+        padding = [math.floor(kernel_size / 2), math.floor(kernel_size / 2)]
 
     else:
-        tot_padding = dilation * (kernel_size - 1)
-        half_padding = torch.div(tot_padding, 2, rounding_mode="floor")
-
-        padding = [half_padding, half_padding]
+        L_out = (
+            math.floor((L_in - dilation * (kernel_size - 1) - 1) / stride) + 1
+        )
+        padding = [
+            math.floor((L_in - L_out) / 2),
+            math.floor((L_in - L_out) / 2),
+        ]
     return padding
 
 
