@@ -25,14 +25,20 @@ logger = logging.getLogger(__name__)
 
 
 def transducer_loss(
-    log_probs, targets, input_lens, target_lens, blank_index, reduction="mean"
+    log_probs,
+    targets,
+    input_lens,
+    target_lens,
+    blank_index,
+    reduction="mean",
+    use_torchaudio=True,
 ):
-    """Transducer loss, see `speechbrain/nnet/transducer/transducer_loss.py`.
+    """Transducer loss, see `speechbrain/nnet/loss/transducer_loss.py`.
 
     Arguments
     ---------
     predictions : torch.Tensor
-        Predicted tensor, of shape [batch, time, chars].
+        Predicted tensor, of shape [batch, maxT, maxU, num_labels].
     targets : torch.Tensor
         Target tensor, without any blanks, of shape [batch, target_len].
     input_lens : torch.Tensor
@@ -40,17 +46,41 @@ def transducer_loss(
     target_lens : torch.Tensor
         Length of each target sequence.
     blank_index : int
-        The location of the blank symbol among the character indexes.
+        The location of the blank symbol among the label indices.
     reduction : str
         Specifies the reduction to apply to the output: 'mean' | 'batchmean' | 'sum'.
+    use_torchaudio: bool
+        If True, use Transducer loss implementation from torchaudio, otherwise,
+        use Speechbrain Numba implementation.
     """
-    from speechbrain.nnet.loss.transducer_loss import Transducer
+    input_lens = (input_lens * log_probs.shape[1]).round().int()
+    target_lens = (target_lens * targets.shape[1]).round().int()
 
-    input_lens = (input_lens * log_probs.shape[1]).int()
-    target_lens = (target_lens * targets.shape[1]).int()
-    return Transducer.apply(
-        log_probs, targets, input_lens, target_lens, blank_index, reduction
-    )
+    if use_torchaudio:
+        try:
+            from torchaudio.functional import rnnt_loss
+        except ImportError:
+            err_msg = "The dependency torchaudio >= 0.10.0 is needed to use Transducer Loss\n"
+            err_msg += "Cannot import torchaudio.functional.rnnt_loss.\n"
+            err_msg += "To use it, please install torchaudio >= 0.10.0\n"
+            err_msg += "==================\n"
+            err_msg += "Otherwise, you can use our numba implementation, set `use_torchaudio=False`.\n"
+            raise ImportError(err_msg)
+
+        return rnnt_loss(
+            log_probs,
+            targets.int(),
+            input_lens,
+            target_lens,
+            blank=blank_index,
+            reduction=reduction,
+        )
+    else:
+        from speechbrain.nnet.loss.transducer_loss import Transducer
+
+        return Transducer.apply(
+            log_probs, targets, input_lens, target_lens, blank_index, reduction,
+        )
 
 
 class PitWrapper(nn.Module):
@@ -130,7 +160,7 @@ class PitWrapper(nn.Module):
         Returns
         -------
         loss : torch.Tensor
-            Permutation invariant loss forthe  current example, tensor of shape [1]
+            Permutation invariant loss for the current example, tensor of shape [1]
 
         assigned_perm : tuple
             Indexes for optimal permutation of the input over sources which
@@ -233,8 +263,8 @@ def ctc_loss(
         See pytorch for 'mean', 'sum', 'none'. The 'batch' option returns
         one loss per item in the batch, 'batchmean' returns sum / batch size.
     """
-    input_lens = (input_lens * log_probs.shape[1]).int()
-    target_lens = (target_lens * targets.shape[1]).int()
+    input_lens = (input_lens * log_probs.shape[1]).round().int()
+    target_lens = (target_lens * targets.shape[1]).round().int()
     log_probs = log_probs.transpose(0, 1)
 
     if reduction == "batchmean":
@@ -828,7 +858,7 @@ class AngularMargin(nn.Module):
         Arguments
         ---------
         outputs : torch.Tensor
-            The outputs of shape [N, C], cosine simiarity is required.
+            The outputs of shape [N, C], cosine similarity is required.
         targets : torch.Tensor
             The targets of shape [N, C], where the margin is applied for.
 
@@ -884,7 +914,7 @@ class AdditiveAngularMargin(AngularMargin):
         Arguments
         ---------
         outputs : torch.Tensor
-            The outputs of shape [N, C], cosine simiarity is required.
+            The outputs of shape [N, C], cosine similarity is required.
         targets : torch.Tensor
             The targets of shape [N, C], where the margin is applied for.
 
@@ -995,7 +1025,7 @@ def ctc_loss_kd(log_probs, targets, input_lens, blank_index, device):
         # Getting current predictions
         current_pred = predictions[j]
 
-        actual_size = (input_lens[j] * log_probs.shape[1]).int()
+        actual_size = (input_lens[j] * log_probs.shape[1]).round().int()
         current_pred = current_pred[0:actual_size]
         current_pred = filter_ctc_output(
             list(current_pred.cpu().numpy()), blank_id=blank_index
@@ -1017,7 +1047,7 @@ def ctc_loss_kd(log_probs, targets, input_lens, blank_index, device):
     fake_lab_lengths = torch.from_numpy(np.array(pred_len_list)).int()
     fake_lab_lengths.to(device)
 
-    input_lens = (input_lens * log_probs.shape[1]).int()
+    input_lens = (input_lens * log_probs.shape[1]).round().int()
     log_probs = log_probs.transpose(0, 1)
     return torch.nn.functional.ctc_loss(
         log_probs,
