@@ -160,10 +160,14 @@ def torch_parameter_transfer(obj, path, device):
 DEFAULT_LOAD_HOOKS = {
     torch.nn.Module: torch_recovery,
     torch.optim.Optimizer: torch_recovery,
+    torch.optim.lr_scheduler._LRScheduler: torch_recovery,
+    torch.optim.lr_scheduler.ReduceLROnPlateau: torch_recovery,
 }
 DEFAULT_SAVE_HOOKS = {
     torch.nn.Module: torch_save,
     torch.optim.Optimizer: torch_save,
+    torch.optim.lr_scheduler._LRScheduler: torch_save,
+    torch.optim.lr_scheduler.ReduceLROnPlateau: torch_save,
 }
 DEFAULT_TRANSFER_HOOKS = {
     torch.nn.Module: torch_parameter_transfer,
@@ -494,7 +498,7 @@ class Checkpointer:
             self.recoverables.update(recoverables)
         else:
             rec = repr(recoverables)  # noqa: F841, rec is used in MSG
-            MSG = "Checkpointer needs a mapping (e.g. dict), \
+            MSG = f"Checkpointer needs a mapping (e.g. dict), \
                     got {rec} instead."
             raise AttributeError(MSG)
 
@@ -1088,6 +1092,7 @@ def average_checkpoints(
     recoverable_name,
     parameter_loader=torch.load,
     averager=average_state_dicts,
+    device=None,
 ):
     """Average parameters from multiple checkpoints.
 
@@ -1149,8 +1154,17 @@ def average_checkpoints(
     tensor([8.])
     """
 
-    parameter_iterator = (
-        parameter_loader(ckpt.paramfiles[recoverable_name])
-        for ckpt in checkpoint_list
-    )
+    try:
+        # try to map the ckps to the correct device
+        parameter_iterator = (
+            parameter_loader(
+                ckpt.paramfiles[recoverable_name], map_location=device
+            )
+            for ckpt in checkpoint_list
+        )
+    except TypeError:
+        parameter_iterator = (
+            parameter_loader(ckpt.paramfiles[recoverable_name])
+            for ckpt in checkpoint_list
+        )
     return averager(parameter_iterator)
