@@ -110,7 +110,7 @@ def ddp_init_group(run_opts):
 
     The ddp group will use distributed_backend arg for setting the
     DDP communication protocol. `RANK` Unix variable will be used for
-    registring the subprocess to the ddp group.
+    registering the subprocess to the ddp group.
 
     Arguments
     ---------
@@ -129,10 +129,7 @@ def ddp_init_group(run_opts):
             if run_opts["local_rank"] + 1 > torch.cuda.device_count():
                 raise ValueError(
                     "Killing process " + str() + "\n"
-                    "To use DDP backend, start your script with:\n\t"
-                    "python -m torch.distributed.launch [args]\n\t"
-                    "experiment.py hyperparams.yaml --distributed_launch=True "
-                    "--distributed_backend=nccl"
+                    "Not enough GPUs available!"
                 )
         if "RANK" in os.environ is None or os.environ["RANK"] == "":
             raise ValueError(
